package compute

import (
	"cloud.o-forge.io/core/oc-lib/dbs"
	"cloud.o-forge.io/core/oc-lib/models/resources/resource_model"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type computeMongoAccessor struct {
	utils.AbstractAccessor // AbstractAccessor contains the basic fields of an accessor (model, caller)
}

// New creates a new instance of the computeMongoAccessor
func New(peerID string, groups []string) *computeMongoAccessor {
	return &computeMongoAccessor{
		utils.AbstractAccessor{
			ResourceModelAccessor: resource_model.New(),
		},
	}
}

/*
* Nothing special here, just the basic CRUD operations
 */

func (dca *computeMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return dca.GenericDeleteOne(id, dca)
}

func (dca *computeMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	set.(*ComputeResource).ResourceModel = nil
	return dca.GenericUpdateOne(set.(*ComputeResource).Trim(), id, dca, &ComputeResource{})
}

func (dca *computeMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	data.(*ComputeResource).ResourceModel = nil
	return dca.GenericStoreOne(data.(*ComputeResource).Trim(), dca)
}

func (dca *computeMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return dca.StoreOne(data)
}

func (dca *computeMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	return utils.GenericLoadOne[*ComputeResource](id, func(d utils.DBObject) (utils.DBObject, int, error) {
		resources, _, err := dca.ResourceModelAccessor.Search(nil, dca.GetType())
		if err == nil && len(resources) > 0 {
			d.(*ComputeResource).ResourceModel = resources[0].(*resource_model.ResourceModel)
		}
		return d, 200, nil
	}, dca)
}

func (wfa *computeMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	resources, _, err := wfa.ResourceModelAccessor.Search(nil, wfa.GetType())
	return utils.GenericLoadAll[*ComputeResource](func(d utils.DBObject, a []utils.ShallowDBObject) []utils.ShallowDBObject {
		if err == nil && len(resources) > 0 {
			d.(*ComputeResource).ResourceModel = resources[0].(*resource_model.ResourceModel)
		}
		a = append(a, d) // only get the abstract resource !
		return a
	}, wfa)
}

func (wfa *computeMongoAccessor) Search(filters *dbs.Filters, search string) ([]utils.ShallowDBObject, int, error) {
	root := &ComputeResource{}
	resources, _, err := wfa.ResourceModelAccessor.Search(nil, wfa.GetType())
	return utils.GenericSearch[*ComputeResource](filters, search, root.GetResourceFilter(search),
		func(d utils.DBObject, a []utils.ShallowDBObject) []utils.ShallowDBObject {
			if err == nil && len(resources) > 0 {
				d.(*ComputeResource).ResourceModel = resources[0].(*resource_model.ResourceModel)
			}
			a = append(a, d) // only get the abstract resource !
			return a
		}, wfa)
}
